/*
 *	KQAESend.cp
 *
 *	"AESend" extension for KeyQuencer
 *
 *	Copyright  1996-1997 Marco Piovanelli
 *	All Rights Reserved
 *
 */


#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __PROCESSES__
#include <Processes.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __AEOBJECTS__
#include <AEObjects.h>
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#include "Extension.h"
#include "Action.h"

//	special signatures

const OSType kThisProcess = 'this' ;

//	delimiters

const char kObjSpecDelimiter = 0xA5 ;
const char kExplicitTypeDelimiter = 0xC5 ;

//	function prototypes

void BlockClear ( void * blockPtr, Size blockSize ) ;
void InitDesc ( AEDesc * desc ) ;
void ForgetDesc ( AEDesc * desc ) ;
void UnquoteParam ( ConstStr255Param srcStr, StringPtr dstStr, GluePtr glue ) ;
int FindChar ( ConstStr255Param str, unsigned char c ) ;
void Extract ( ConstStr255Param str, int start, int stop, StringPtr substr ) ;
DescType ExtractExplicitType ( StringPtr str ) ;
DescType StringToDescType ( ConstStr255Param str ) ;
OSErr ParseSpecifier ( ConstStr255Param str, AEDesc * desc ) ;
OSErr CreateObjectSpecifier ( const AEDesc * inContainerSpec, DescType inDesiredClass, DescType inKeyForm, const AEDesc * inKeyData, AEDesc * outResult ) ;

inline void CopyPString ( ConstStr255Param srcString, StringPtr dstString )
{
	BlockMoveData ( srcString, dstString, StrLength ( srcString ) + 1 ) ;
}

SInt16 run ( ParamsPtr params, MachineHandle mac, GluePtr glue )
{
	//	this is where the extension does its work
	//	cur heap zone = active app heap, cur res file = unknown
	//	the extension file is closed, resources are not available

	Str255 param ;
	DescType signature ;
	DescType keyword ;
	DescType explicitType ;
	AEEventClass eventClass ;
	AEEventID eventID ;
	AEAddressDesc address ;
	AEDesc desc ;
	AppleEvent ae ;
	THz saveZone ;
	SInt16 paramIndex ;
	OSErr err ;

	InitDesc ( & address ) ;
	InitDesc ( & ae ) ;
	InitDesc ( & desc ) ;

	//	set the zone to the system zone
	saveZone = GetZone ( ) ;
	SetZone ( SystemZone ( ) ) ;

	//	we need KeyQuencer 2.0 or newer (glue version = 6)
	err = kTellObsoleteKeyQuencer ;
	if ( glue -> glueRecVers < 6 )
	{
		goto cleanup ;
	}

	//	we need Apple events!
	err = kTellRequiresAppleEvents ;
	if ( ! ( * mac ) -> hasAppleEvents )
	{
		goto cleanup ;
	}

	//	make sure we got the needed parameters
	err = kTellNotEnoughParams ;
	if ( params -> paramsCount < 3 )
	{
		goto cleanup ;
	}

	//	parameters must be an odd number
	if ( params -> paramsCount & 0x0001 == 0 )
	{
		goto cleanup ;
	}

	//	extract application signature from parameter #0
	err = kTellBadParameters ;
	UnquoteParam ( params -> parameter [ 0 ], param, glue ) ;
	if ( ( signature = StringToDescType ( param ) ) == 0 )
	{
		goto cleanup ;
	}

	//	the special signature 'this' means current process
	if ( signature == kThisProcess )
	{
		ProcessSerialNumber psn ;
		if ( ( err = GetCurrentProcess ( & psn ) ) != noErr )
		{
			goto cleanup ;
		}
		
		//	create an address descriptor based on the PSN
		if ( ( err = AECreateDesc ( typeProcessSerialNumber, & psn, sizeof ( psn ), & address ) ) != noErr )
		{
			goto cleanup ;
		}
	}
	else
	{
		//	create an address descriptor based on the app signature
		if ( ( err = AECreateDesc ( typeApplSignature, & signature, sizeof ( signature ), & address ) ) != noErr )
		{
			goto cleanup ;
		}
	}

	//	extract event class from parameter #1
	err = kTellBadParameters ;
	UnquoteParam ( params -> parameter [ 1 ], param, glue ) ;
	if ( ( eventClass = StringToDescType ( param ) ) == 0 )
	{
		goto cleanup ;
	}

	//	extract event ID from parameter #2
	UnquoteParam ( params -> parameter [ 2 ], param, glue ) ;
	if ( ( eventID = StringToDescType ( param ) ) == 0 )
	{
		goto cleanup ;
	}

	//	create the Apple event
	if ( ( err = AECreateAppleEvent ( eventClass, eventID, & address, kAutoGenerateReturnID, kAnyTransactionID, & ae ) ) != noErr )
	{
		goto cleanup ;
	}

	//	check for parameters
	for ( paramIndex = 3 ; paramIndex < params -> paramsCount ; paramIndex += 2 )
	{
		err = kTellBadParameters ;

		//	extract keyword
		UnquoteParam ( params -> parameter [ paramIndex ], param, glue ) ;

		//	'-' is short for '----'
		if ( ( StrLength ( param ) == 1 ) && ( param [ 1 ] == '-' ) )
		{
			keyword = '----' ;
		}
		else if ( ( keyword = StringToDescType ( param ) ) == 0 )
		{
			goto cleanup ;
		}

		//	extract Apple event parameter
		UnquoteParam ( params -> parameter [ paramIndex + 1 ], param, glue ) ;

		//	look for an explicit type code appended at the end
		explicitType = ExtractExplicitType ( param ) ;

		//	is this parameter an object specifier?
		if ( FindChar ( param, kObjSpecDelimiter ) != 0 )
		{
			//	parse the specifier
			if ( ( err = ParseSpecifier ( param, & desc ) ) != noErr )
			{
				goto cleanup ;
			}
		}
		else
		{
			//	generic TEXT parameter
			if ( ( err = AECreateDesc ( typeChar, & param [ 1 ], StrLength ( param ), & desc ) ) != noErr )
			{
				goto cleanup ;
			}
		}

		//	if an explicit type was specified, coerce the parameter
		if ( explicitType != 0 )
		{
			AEDesc coercedDesc ;

			InitDesc ( & coercedDesc ) ;
			if ( ( err = AECoerceDesc ( & desc, explicitType, & coercedDesc ) ) != noErr )
			{
				goto cleanup ;
			}

			ForgetDesc ( & desc ) ;
			desc = coercedDesc ;
		}

		//	add parameter to Apple event
		if ( ( err = AEPutParamDesc ( & ae, keyword, & desc ) ) != noErr )
		{
			goto cleanup ;
		}
	}

	//	send the event through the KeyQuencer glue
	err = errAEEventNotHandled ;
	if ( ! glue -> SendAppleEvent ( & ae, nil, false ) )
	{
		goto cleanup ;
	}

	//	clear result code
	err = noErr ;

cleanup :
	//	dispose of temporary structures
	ForgetDesc ( & address ) ;
	ForgetDesc ( & desc ) ;

	//	restore the original zone
	SetZone ( saveZone ) ;

	//	return result code
	return err ;
}

SInt16 init ( ParamsPtr params, MachineHandle mac, GluePtr glue )
{
#pragma unused ( params, mac, glue )

	// this only gets called once at startup time
	// cur heap zone = sys heap, cur res file = extension res file
	// you may load (and detach) resources and initialize your data

	return kExtNoError ;
}

void BlockClear ( void * blockPtr, register Size blockSize )
{
	register char * p = ( char * ) blockPtr ;

	while ( -- blockSize >= 0 )
	{
		* p ++ = 0 ;
	}
}

void InitDesc ( AEDesc * desc )
{
	desc -> descriptorType = typeNull ;
	desc -> dataHandle = nil ;
}

void ForgetDesc ( AEDesc * desc )
{
	if ( ( desc -> descriptorType != typeNull ) || ( desc -> dataHandle != nil ) )
	{
		AEDisposeDesc ( desc ) ;
	}
}

void UnquoteParam ( ConstStr255Param srcStr, StringPtr dstStr, GluePtr glue )
{
	if ( ! glue -> IsQuotedParameter ( srcStr, dstStr, 255, kKQSingleQuote ) )
	{
		CopyPString ( srcStr, dstStr ) ;
	}
}

DescType StringToDescType ( ConstStr255Param str )
{
	DescType tmp = 0 ;

	if ( StrLength ( str ) == sizeof ( tmp ) )
	{
		BlockMoveData ( & str [ 1 ], & tmp, sizeof ( tmp ) ) ;
	}

	return tmp ;
}

OSErr CreateObjectSpecifier
	(
		const AEDesc * inContainerSpec ,
		DescType inDesiredClass ,
		DescType inKeyForm ,
		const AEDesc * inKeyData ,
		AEDesc * outResult
	)
{
	AEDesc recordDesc ;
	OSErr err ;

	InitDesc ( outResult ) ;
	InitDesc ( & recordDesc ) ;

	//	create an AE record
	if ( ( err = AECreateList ( nil, 0, true, & recordDesc ) ) != noErr )
	{
		goto cleanup ;
	}

	//	add the keyword-specified fields
	if ( ( err = AEPutParamDesc ( & recordDesc, keyAEContainer, inContainerSpec ) ) != noErr )
	{
		goto cleanup ;
	}
	if ( ( err = AEPutParamPtr ( & recordDesc, keyAEDesiredClass, typeType, & inDesiredClass, sizeof ( inDesiredClass ) ) ) != noErr )
	{
		goto cleanup ;
	}
	if ( ( err = AEPutParamPtr ( & recordDesc, keyAEKeyForm, typeEnumerated, & inKeyForm, sizeof ( inKeyForm ) ) ) != noErr )
	{
		goto cleanup ;
	}
	if ( ( err = AEPutParamDesc ( & recordDesc, keyAEKeyData, inKeyData ) ) != noErr )
	{
		goto cleanup ;
	}

	//	coerce the record to typeObjectSpecifier
	if ( ( err = AECoerceDesc ( & recordDesc, typeObjectSpecifier, outResult ) ) != noErr )
	{
		goto cleanup ;
	}

	//	clear result code
	err = noErr ;

cleanup :
	ForgetDesc ( & recordDesc ) ;
	return err ;
}

int FindChar ( ConstStr255Param str, unsigned char c )
{
	int len = StrLength ( str ) ;

	for ( int i = 1 ; i <= len ; i ++ )
	{
		if ( str [ i ] == c )
		{
			return i ;
		}
	}
	return 0 ;
}

void Extract ( ConstStr255Param str, int start, int stop, StringPtr substr )
{
	int i, j ;
	int len = StrLength ( str ) ;

	for ( i = start, j = 0 ; ( i <= stop ) && ( i <= len ) ; i ++ )
	{
		substr [ ++ j ] = str [ i ] ;
	}
	substr [ 0 ] = j ;
}

DescType ExtractExplicitType ( StringPtr str )
{
	unsigned char substr [ 5 ] ;
	int i ;

	i = FindChar ( str, kExplicitTypeDelimiter ) ;
	if ( ( i > 0 ) && ( i + 4 == StrLength ( str ) ) )
	{
		Extract ( str, i + 1, 255, substr ) ;
		Extract ( str, 1, i - 1, str ) ;
		return StringToDescType ( substr ) ;
	}

	return 0 ;
}

OSErr ParseSpecifier ( ConstStr255Param str, AEDesc * desc )
{
	int i ;
	Str255 s1, s2 ;
	DescType desiredClass ;
	DescType keyForm ;
	AEDesc keyData ;
	AEDesc tempDesc ;
	DescType tag ;
	SInt32 index ;
	OSErr err ;

	InitDesc ( desc ) ;
	InitDesc ( & keyData ) ;
	InitDesc ( & tempDesc ) ;

	CopyPString ( str, s1 ) ;

	do
	{
		if ( ( i = FindChar ( s1, kObjSpecDelimiter ) ) == 0 )
		{
			i = StrLength ( s1 ) ;
		}

		if ( i < 4 )
		{
			err = kTellBadParameters ;
			goto cleanup ;
		}

		if ( ( i > 5 ) && ( FindChar ( s1, '[' ) != 5 ) && ( FindChar ( s1, ']' ) < i - 1 ) )
		{
			goto cleanup ;
		}

		//	extract desired class code (first four characters)
		Extract ( s1, 1, 4, s2 ) ;
		desiredClass = StringToDescType ( s2 ) ;

		//	extract key data (characters between brackets)
		Extract ( s1, 6, FindChar ( s1, ']' ) - 1, s2 ) ;

		//	heuristically determine key form
		if ( i <= 5 )
		{
			//	property form
			keyForm = formPropertyID ;
			tag = desiredClass ;
			desiredClass = cProperty ;
			if ( ( err = AECreateDesc ( typeType, & tag, sizeof ( tag), & keyData ) ) != noErr )
			{
				goto cleanup ;
			}
		}
		else if ( ( ( s2 [ 1 ] >= '0' ) && ( s2 [ 1 ] <= '9' ) ) || ( s2 [ 1 ] == '-' ) )
		{
			//	index form
			keyForm = formAbsolutePosition ;
			StringToNum ( s2, & index ) ;
			if ( ( err = AECreateDesc ( typeInteger, & index, sizeof ( index ), & keyData ) ) != noErr )
			{
				goto cleanup ;
			}
		}
		else if ( ( StrLength ( s2 ) > 1 ) && ( s2 [ 1 ] == '_' ) )
		{
			//	ID form
			keyForm = formUniqueID ;
			Extract ( s2, 2, 255, s2 ) ;
			if ( ( err = AECreateDesc ( typeChar, & s2 [ 1 ], StrLength ( s2 ), & keyData ) ) != noErr )
			{
				goto cleanup ;
			}
		}
		else if ( ( StrLength ( s2 ) == 1 ) && ( s2 [ 1 ] == '*' ) )
		{
			//	'*' means 'all'
			keyForm = formAbsolutePosition ;
			tag = kAEAll ;
			if ( ( err = AECreateDesc ( typeAbsoluteOrdinal, & tag, sizeof ( tag ), & keyData ) ) != noErr )
			{
				goto cleanup ;
			}
		}
		else
		{
			//	name form
			keyForm = formName ;
			if ( ( err = AECreateDesc ( typeChar, & s2 [ 1 ], StrLength ( s2 ), & keyData ) ) != noErr )
			{
				goto cleanup ;
			}
		}

		//	build object specifier
		if ( ( err = CreateObjectSpecifier ( desc, desiredClass, keyForm, & keyData, & tempDesc ) ) != noErr )
		{
			goto cleanup ;
		}

		//	the newly built specifier becomes the next container
		ForgetDesc ( desc ) ;
		ForgetDesc ( & keyData ) ;
		* desc = tempDesc ;
		InitDesc ( & tempDesc ) ;

		//	parse the next specifier
		Extract ( s1, i + 1, 255, s1 ) ;
	} while ( StrLength ( s1 ) > 0 ) ;

	//	clear result code
	err = noErr ;

cleanup :
	ForgetDesc ( & keyData ) ;
	ForgetDesc ( & tempDesc ) ;
	return err ;
}
